'use strict';

var _asyncChildProcess = require('async-child-process');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _loaderUtils = require('loader-utils');

var _loaderUtils2 = _interopRequireDefault(_loaderUtils);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const findSrcDir = (() => {
  var _ref = _asyncToGenerator(function* (childPath) {
    let candidate = childPath;

    while (candidate !== _path2.default.parse(candidate).root) {
      const maybeCargoFile = _path2.default.join(candidate, 'Cargo.toml');
      if (yield _fsExtra2.default.pathExists(maybeCargoFile)) {
        return candidate;
      }
      candidate = _path2.default.dirname(candidate);
    }

    return null;
  });

  return function findSrcDir(_x) {
    return _ref.apply(this, arguments);
  };
})();

const load = (() => {
  var _ref2 = _asyncToGenerator(function* (self) {
    const srcDir = yield findSrcDir(self.resourcePath);
    if (!srcDir) {
      throw new Error('No Cargo.toml file found in any parent directory.');
    }

    const opts = _loaderUtils2.default.getOptions(self);
    const release = opts && opts.hasOwnProperty('release') ? opts['release'] : false;
    const gc = opts && opts.hasOwnProperty('gc') ? opts['gc'] : false;
    const target = opts && opts.hasOwnProperty('target') ? opts['target'] : 'wasm32-unknown-unknown';

    const cmd = `cargo build --message-format=json --target=${target}${release ? ' --release' : ''}`;

    const result = yield (0, _asyncChildProcess.execAsync)(cmd, { cwd: srcDir });

    let wasmFile;
    for (let line of result.stdout.split(_os2.default.EOL)) {
      if (/^\s*$/.test(line)) {
        continue;
      }
      const data = JSON.parse(line);
      if (data.hasOwnProperty('filenames')) {
        wasmFile = data['filenames'].find(function (p) {
          return p.endsWith('.wasm');
        });
        break;
      }
    }

    if (!wasmFile) {
      throw new Error('No wasm file produced as build output', null);
    }

    if (gc) {
      let gcWasmFile = wasmFile.substr(0, wasmFile.length - '.wasm'.length) + '.gc.wasm';
      yield (0, _asyncChildProcess.execAsync)(`wasm-gc ${wasmFile} ${gcWasmFile}`);
      wasmFile = gcWasmFile;
    }

    return yield _fsExtra2.default.readFile(wasmFile);
  });

  return function load(_x2) {
    return _ref2.apply(this, arguments);
  };
})();

module.exports = function () {
  const callback = this.async();
  load(this).then(r => callback(null, r), e => callback(e, null));
};